<?php

namespace App\Controller;

use App\Entity\Commande;
use App\Entity\Librairie;
use App\Entity\Service;
use App\Entity\Statut;
use App\Entity\Tape;
use App\Services\LotService;
use App\Services\TapeService;
use App\Entity\TapeModele;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use App\Entity\Lot;

class LotController extends  AbstractController
{
    /**
     * @Route("/listLots",name="show_lots")
     */
    public function listLots(Request $request)
    {
        $tapes=null;

        if ($request->query->get("lotid")){
            $Lots = $this->getDoctrine()
                ->getRepository(Lot::class)
                ->find($request->query->get("lotid"));

            $tapes =$this->getDoctrine()
                ->getRepository(Tape::class)
                ->findBy(array('lotid'=>$request->query->get("lotid")));

        }
        else {
            $Lots = $this->getDoctrine()
                ->getRepository(Lot::class)
                ->findAll();

        }
            return $this->render('lots/main.html.twig', array('lots' => $Lots,'tapes'=>$tapes));

    }

    /**
     * @Route("/addLots", name="add_Lots")
     */
    public function addLots(Request $request, ManagerRegistry $doctrine, LotService $lotService, TapeService $tapeService){

        $entityManager = $doctrine->getManager();

        // Données relatives à la commande
        $paramCommandes = $request->request->get('commande');

        //Données relatives aux lots
        $paramModel = $request->request->get('model');
        $paramLabel = $request->request->get('label');
        $paramIndiceStart = $request->request->get('indicedebut');
        $paramNbTapes = $request->request->get('nbtapes');
        $paramService = $request->request->get('service');
        $paramLibrairies = $request->request->get('librairie');
        $paramPrix = $request->request->get('prix');
        $paramComments = $request->request->get('commentaire');

        //  $nbtapes=(int) $indice2-$indice1;

        $statusTape = $doctrine->getRepository(Statut::class)->find(8);
        $statusLot = $doctrine->getRepository(Statut::class)->find(36);

        /** @var TYPE_NAME $exception */
        try {

            for ($i = 0; $i < count($paramModel); $i++) {

                $commandeId = $paramCommandes[$i];
                $modeleId = $paramModel[$i];
                $nbTapes = $paramNbTapes[$i];
                $serviceId = $paramService[$i];
                $libraryId = $paramLibrairies[$i];

                $lot = $lotService->create($commandeId, $modeleId, $statusLot, $nbTapes);

                $entityManager->persist($lot);

                $service = $doctrine->getRepository(Service::class)->find($serviceId);
                $librairie = $doctrine->getRepository(Librairie::class)->find($libraryId);

                /*Creation des tapes */

                for ($j = $paramIndiceStart[$i]; $j < $paramIndiceStart[$i] + $paramNbTapes[$i]; $j++) {

                    $prix = $paramPrix[$i];
                    $commentaire = $paramComments[$i];
                    $label = $paramLabel[$i];

                    $Tape = $tapeService->create($lot, $statusTape, $service, $librairie, $prix, $commentaire, $label, $j);

                    $entityManager->persist($Tape);
                }

            }
            $entityManager->flush();
            $entityManager->clear();

            $response = new Response(json_encode(array(
                'status' => '200',
                'message' => 'Base de données mise à jour'
            )));

        }
        catch (DBALException $e) {

            $response = new Response(json_encode(array(
                'status' => '500',
                'message' => $e
            )));
        }

        $response = new Response(json_encode(array(
            'status' => '200',
            'message' => 'Base de données mise à jour'
        )));
        $response->headers->set('Content-Type', 'application/json');
        return $response;

    }

    /**
     * @Route("/regenerateLot", name="regenerate_Lot")
     */
    public function regenerateLot(Request $request, ManagerRegistry $doctrine, LotService $lotService, TapeService $tapeService){

        $entityManager = $doctrine->getManager();

        // Données relatives à la commande
        $paramCommandes = $request->request->get('commande')[0];

        //Données relatives aux lots
        $paramLot = $request->request->get('idLot');
        $paramModel = $request->request->get('model')[0];
        $paramLabel = $request->request->get('label')[0];
        $paramIndiceStart = $request->request->get('indicedebut')[0];
        $paramNbTapes = $request->request->get('nbtapes')[0];
        $paramService = $request->request->get('service')[0];
        $paramLibrairies = $request->request->get('librairie')[0];
        $paramPrix = $request->request->get('prix')[0];
        $paramComments = $request->request->get('commentaire')[0];

        /*** Remove current Lot and nested tapes ***/

        $lot = $doctrine->getRepository(Lot::class)->find($paramLot);
        $tapes = $doctrine->getRepository(Tape::class)->findBy(['lotid' => $lot->getLotid()]);

        foreach($tapes as $tape) {
            $entityManager->remove($tape);
        }

        $entityManager->remove($lot);
        $entityManager->flush();

        /*** Generate the new Lot and nested tapes ***/

        $statusTape = $doctrine->getRepository(Statut::class)->find(8);
        $statusLot = $doctrine->getRepository(Statut::class)->find(36);

        /** @var TYPE_NAME $exception */
        try {

            $commandeId = $paramCommandes;
            $modeleId = $paramModel;
            $nbTapes = $paramNbTapes;
            $serviceId = $paramService;
            $libraryId = $paramLibrairies;

            $lot = $lotService->create($commandeId, $modeleId, $statusLot, $nbTapes);

            $entityManager->persist($lot);

            $service = $doctrine->getRepository(Service::class)->find($serviceId);
            $librairie = $doctrine->getRepository(Librairie::class)->find($libraryId);

            /*Creation des tapes */

            for ($j = $paramIndiceStart; $j < $paramIndiceStart + $paramNbTapes; $j++) {

                $prix = $paramPrix;
                $commentaire = $paramComments;
                $label = $paramLabel;

                $Tape = $tapeService->create($lot, $statusTape, $service, $librairie, $prix, $commentaire, $label, $j);

                $entityManager->persist($Tape);
            }

            $entityManager->flush();
            $entityManager->clear();

            $response = new Response(json_encode(array(
                'status' => '200',
                'message' => 'Base de données mise à jour'
            )));

        }
        catch (DBALException $e) {

            $response = new Response(json_encode(array(
                'status' => '500',
                'message' => $e
            )));
        }

        $response = new Response(json_encode(array(
            'status' => '200',
            'message' => 'Base de données mise à jour'
        )));
        $response->headers->set('Content-Type', 'application/json');
        return $response;
    }

    /**
     * @Route("/editLot", name="edit_lot")
     */
    public function editLot(Request $request){



        $em = $this->getDoctrine()->getManager();

        /** @var TYPE_NAME $exception */
        try {

            $id=$request->request->get('id');
            $em = $this->getDoctrine()->getManager();

            $lot=$this->getDoctrine()
                    ->getRepository(Lot::class)->find($id);


            $em->persist($lot);
            $em->flush();

            $response = new Response(json_encode(array(
                'status' => '200',
                'message' => 'Base de données mise à jour'
            )));

        }
        catch (DBALException $e) {

            $response = new Response(json_encode(array(
                'status' => '500',
                'message' => $e
            )));
        }


        $response->headers->set('Content-Type', 'application/json');
        return $response;

    }

    /**
     * @Route("/changeLotStatus", name="change_lot_status")
     */

    public function changeLotStatus(Request $request, ManagerRegistry $doctrine) {

        $statutId = $request->request->get('statutid');
        $date = $request->request->get('date');

        $datetime = new \DateTime($date);

        $em = $doctrine->getManager();
        $lot = $doctrine
            ->getRepository(Lot::class)
            ->find($request->request->get("lotid"));

        $statusLot = $doctrine->getRepository(Statut::class)->find( $statutId);

        $lot->setStatutId($statusLot);

        if ($statutId==38) {
            $statutTid=11;
            $statutTape = $doctrine->getRepository(Statut::class)->find($statutTid);
            $lot->setDateLivraison($datetime);
        }
        if ($statutId==40) {
            $statutTid=20;
            $statutTape = $doctrine->getRepository(Statut::class)->find($statutTid);
            $lot->setDateMiseService($datetime);

        }

        $tapes = $doctrine
            ->getRepository(Tape::class)
            ->findBy(array('lotid'=>$request->request->get("lotid")));

            foreach ($tapes as $tape) {

                $arr=json_decode($tape->getHistory(),TRUE);
                $arr[]=array($statutTid => $datetime);
                $tape->setHistory(json_encode($arr));
                $tape->setStatutId($statutTape);
                $em->persist($tape);
            }
            $em->flush();

        $response = new Response(json_encode(array(
            'status' => '200',
            'message' => 'Base de données mise à jour'
        )));
        $response->headers->set('Content-Type', 'application/json');
        return $response;

        }
}
