<?php


namespace App\Controller;

use App\Entity\Service;
use App\Entity\Statut;
use App\Entity\Tape;
use App\Entity\Librairie;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use App\Entity\TapeModele;
use Symfony\Component\HttpFoundation\JsonResponse;
use JMS\Serializer\SerializerInterface;
use Doctrine\ORM\Tools\Pagination\Paginator;

class TapeController extends AbstractController
{
    /**
     * @Route("/listModels",name="show_models")
     */
    public function listModels(Request $request)
    {
        $models = $this->getDoctrine()
            ->getRepository(TapeModele::class)
            ->findAll();

        return $this->render('tapes/models.html.twig', array('models' => $models));
    }

    /**
     * @Route("/listTapes",name="show_tapes")
     */
    public function listTapes(Request $request, ManagerRegistry $doctrine, SerializerInterface $serializer)
    {
//        $em=$this->getDoctrine()->getManager();
//        $tapes=array();
//
//
//        $q = $em->createQuery('SELECT t.tapeid , t.label ,t.datecreation , s.statut , s.statutid , m.nom as modele, e.nom as service ,l.nom as librairie from App\Entity\Tape t, App\Entity\Statut s , App\Entity\Librairie l , App\Entity\TapeModele m , App\Entity\Service e  where t.serviceid=e.serviceid and t.statutid=s.statutid and t.modeleid=m.modelid');
//        $iterableResult = $q->iterate();
//        foreach ($iterableResult as $row) {
//            foreach ($row as $tape)
//                $tapes[]=$tape;
//        }
        /*$tapes = $this->getDoctrine()
            ->getRepository(Tape::class)
            ->findAll();*/

        $tabModeles = $doctrine->getRepository(TapeModele::class)
            ->findBy([], ['nom' => 'ASC']);

        $tabStatuts = $doctrine->getRepository(Statut::class)
            ->findBy([], ['statut' => 'ASC']);

        $tabServices = $doctrine->getRepository(Service::class)
            ->findBy([], ['nom' => 'ASC']);

        $tabLibrairies = $doctrine->getRepository(Librairie::class)
            ->findBy([], ['nom' => 'ASC']);

        return $this->render('tapes/main.html.twig', [
            'tabModelesJson'=> $serializer->serialize($tabModeles, 'json'),
            'tabServicesJson'=> $serializer->serialize($tabServices, 'json'),
            'tabStatutsJson' => $serializer->serialize($tabStatuts, 'json'),
            'tabLibrairiesJson' => $serializer->serialize($tabLibrairies, 'json'),
            'tabServices'=> $tabServices,
            'tabStatuts' => $tabStatuts,
        ]);
    }

    /**
     * @Route("/ajaxTapesContent",name="ajax_tapes_content")
     */
    public function ajaxTapesContent(Request $request, ManagerRegistry $doctrine, SerializerInterface $serializer)
    {
        $params = $request->request->all();
        $colums = $params['columns'];
        $draw = $params['draw'];
        $start = $params['start'];
        $length = $params['length'];
        $order = $params['order'];
        $orderColumn = $order[0]['column'];
        $orderDir = $order[0]['dir'];
        // Raw text search value
        $search = $params['search']['value'];

        $tapeRepository = $doctrine->getManager()->getRepository(Tape::class);

        $mainQueryBuilder = $tapeRepository->createQueryBuilder('t')
            ->select('t')
            ->innerJoin('t.modeleid', 'm')
            ->innerJoin('t.statutid', 'st')
            ->innerJoin('t.serviceid', 'se')
            ->innerJoin('t.librairieid', 'li')
            ->setFirstResult( $start )
            ->setMaxResults( $length );

        $countQueryBuilder = $tapeRepository->createQueryBuilder('t')
            ->select('count(t)')
            ->innerJoin('t.modeleid', 'm')
            ->innerJoin('t.statutid', 'st')
            ->innerJoin('t.serviceid', 'se')
            ->innerJoin('t.librairieid', 'li');

        // Dropdown list search
        foreach($colums as $key => $colum) {

            $searchId = $colum['search']['value'];

            if($searchId) {

                if($key == 1) { // modeleid.nom
                    $mainQueryBuilder->andWhere('t.modeleid = :modelId');
                    $mainQueryBuilder->setParameter('modelId', $searchId);
                    $countQueryBuilder->andWhere('t.modeleid = :modelId');
                    $countQueryBuilder->setParameter('modelId', $searchId);
                }

                if($key == 2) { // statutid.statut
                    $mainQueryBuilder->andWhere('t.statutid = :statusId');
                    $mainQueryBuilder->setParameter('statusId', $searchId);
                    $countQueryBuilder->andWhere('t.statutid = :statusId');
                    $countQueryBuilder->setParameter('statusId', $searchId);
                }

                if($key == 3) { // serviceid.nom
                    $mainQueryBuilder->andWhere('se.serviceid = :serviceId');
                    $mainQueryBuilder->setParameter('serviceId', $searchId);
                    $countQueryBuilder->andWhere('se.serviceid = :serviceId');
                    $countQueryBuilder->setParameter('serviceId', $searchId);
                }

                if($key == 4) { // library.nom
                    $mainQueryBuilder->andWhere('li.librairieid = :libraryId');
                    $mainQueryBuilder->setParameter('libraryId', $searchId);
                    $countQueryBuilder->andWhere('li.librairieid = :libraryId');
                    $countQueryBuilder->setParameter('libraryId', $searchId);
                }
            }
        }

        // Raw text search
        if($search) {

            $mainQueryBuilder->orWhere('t.label LIKE :tapeLabel');
            $mainQueryBuilder->setParameter('tapeLabel', $search.'%');
            $countQueryBuilder->orWhere('t.label LIKE :tapeLabel');
            $countQueryBuilder->setParameter('tapeLabel', $search.'%');

            $mainQueryBuilder->orWhere('m.nom LIKE :modelName');
            $mainQueryBuilder->setParameter('modelName', $search.'%');
            $countQueryBuilder->orWhere('m.nom LIKE :modelName');
            $countQueryBuilder->setParameter('modelName', $search.'%');

            $mainQueryBuilder->orWhere('st.statut LIKE :statusName');
            $mainQueryBuilder->setParameter('statusName', $search.'%');
            $countQueryBuilder->orWhere('st.statut LIKE :statusName');
            $countQueryBuilder->setParameter('statusName', $search.'%');

            $mainQueryBuilder->orWhere('se.nom LIKE :serviceName');
            $mainQueryBuilder->setParameter('serviceName', $search.'%');
            $countQueryBuilder->orWhere('se.nom LIKE :serviceName');
            $countQueryBuilder->setParameter('serviceName', $search.'%');

            $mainQueryBuilder->orWhere('li.nom LIKE :libraryName');
            $mainQueryBuilder->setParameter('libraryName', $search.'%');
            $countQueryBuilder->orWhere('li.nom LIKE :libraryName');
            $countQueryBuilder->setParameter('libraryName', $search.'%');

        }

        // Sorting columns
        if (isset($order)) {
            if($orderColumn == 0) // label
                $mainQueryBuilder->orderBy('t.' . $colums[$orderColumn]['data'], $orderDir);
            else if($orderColumn == 1) // modeleid.nom
                $mainQueryBuilder->orderBy('m.nom', $orderDir);
            else if($orderColumn == 2) // statutid.statut
                $mainQueryBuilder->orderBy('st.statut', $orderDir);
            else if($orderColumn == 3) // serviceid.nom
                $mainQueryBuilder->orderBy('se.nom', $orderDir);
            else if($orderColumn == 4) // library.nom
                $mainQueryBuilder->orderBy('li.nom', $orderDir);
            else if($orderColumn == 5) // datecreation
                $mainQueryBuilder->orderBy('t.' . $colums[$orderColumn]['data'], $orderDir);
        }

        $data = $mainQueryBuilder
            ->getQuery()
            ->getResult();

        $count = $countQueryBuilder
            ->getQuery()
            ->getSingleColumnResult()[0];

        $countTotal = $tapeRepository->createQueryBuilder('a')
            ->select('count(a.tapeid)')
            ->getQuery()
            ->getSingleScalarResult();

        $renderDatatable = json_encode([
            "draw" => (int)$draw,
            "recordsTotal" => (int)$countTotal,
            "recordsFiltered" => (int)$count,
            "data" => json_decode($serializer->serialize($data, 'json'))
        ]);

        return new JsonResponse($renderDatatable,
            200,
            [],
            true
        );
    }

    /**
     * @Route("/addModel", name="add_Model")
     */
    public function addModel(Request $request){

        $nom = $request->request->get('nom');
        $fabricant = $request->request->get('fabricant');
        $capacite = $request->request->get('capacite');
        $longueur = $request->request->get('longueur');

        $em = $this->getDoctrine()->getManager();

        /** @var TYPE_NAME $exception */
        try {

            $entityManager = $this->getDoctrine()->getManager();

            $model=new TapeModele();
            $model->setNom($nom);
            $model->setFabricant($fabricant);
            $model->setCapacite($capacite);
            $model->setLongueur($longueur);
            $model->setDatecreation(new \DateTime());
            $entityManager->persist($model);
            $entityManager->flush();

            $response = new Response(json_encode(array(
                'status' => '200',
                'message' => 'Base de données mise à jour'
            )));

        }
        catch (DBALException $e) {

            $response = new Response(json_encode(array(
                'status' => '500',
                'message' => $e
            )));
        }


        $response->headers->set('Content-Type', 'application/json');
        return $response;

    }

    /**
     * @Route("/editModel", name="edit_Model")
     */
    public function editModel(Request $request){

        $id=$request->request->get('id');
        $nom=$request->request->get('nom');
        $fabricant=$request->request->get('fabricant');
        $capacite=$request->request->get('capacite');
        $longueur=$request->request->get('longueur');

        $em = $this->getDoctrine()->getManager();

        /** @var TYPE_NAME $exception */
        try {

            $id=$request->request->get('id');
            $em = $this->getDoctrine()->getManager();

            $model=$this->getDoctrine()
                ->getRepository(TapeModele::class)->find($id);
            $model->setNom($nom);
            $model->setFabricant($fabricant);
            $model->setCapacite($capacite);
            $model->setLongueur($longueur);

            $em->persist($model);
            $em->flush();

            $response = new Response(json_encode(array(
                'status' => '200',
                'message' => 'Base de données mise à jour'
            )));

        }
        catch (DBALException $e) {

            $response = new Response(json_encode(array(
                'status' => '500',
                'message' => $e
            )));
        }


        $response->headers->set('Content-Type', 'application/json');
        return $response;

    }

    /**
     * @Route("/changeStatus", name="change_status")
     */

    public function changeStatus(Request $request) {
        $statutId=$request->request->get('statut');
         $listId = $request->request->get('listIds');

        $em = $this->getDoctrine()->getManager();

        foreach ($listId as $id) {

            $tape=$this->getDoctrine()
                ->getRepository(Tape::class)->find($id);

            $arr=json_decode($tape->getHistory(),TRUE);
            $date=new \DateTime("now");
            $arr[]=array($statutId=>$date);
            $tape->setHistory(json_encode($arr));




        }
        $em->flush();

    }

    /**
     * @Route("/tapeDetails", name="tape_details")
     */

    public function tapeDetails(Request $request, ManagerRegistry $doctrine) {

        $id = $request->request->get('tapeid');

        $em = $doctrine->getManager();
        $tape = $doctrine->getRepository(Tape::class)->find($id);

        $tabStatut = $doctrine->getRepository(Statut::class)->findAll();
        $tabstatuts = array();

        foreach ($tabStatut as $statut) {
            $tabstatuts[$statut->getStatutId()]["statut"] = $statut->getStatut();
            $tabstatuts[$statut->getStatutId()]["color"] = $statut->getColor();
            $tabstatuts[$statut->getStatutId()]["icon"] = $statut->getIcon();

        }

        $histories = json_decode($tape->getHistory(),TRUE);

        $htmlhistory = '<ul class="timeline">';
        $cpt = 0;

        foreach ($histories as $history) {

            foreach ($history as $statut => $item) {

                if ($cpt % 2 == 0)
                    $timeline = 'timeline-inverted ';
                else
                    $timeline = '';

                $htmlhistory .= '<li class="'.$timeline.'">
                                  <div class=" timeline-badge '.$tabstatuts[$statut]['color'] . '"><i class="' . $tabstatuts[$statut]['icon'] . '"></i></div>
                                  <div class="timeline-panel">
                                    <div class="timeline-heading">
                                      <h4 class="timeline-title">' . $tabstatuts[$statut]["statut"] . '</h4>
                                      <p><small class="text-muted"><i class="far fa-clock"></i> ' . substr($item["date"], 0, 16) . '</small></p>
                                    </div>
                                  </div>
                                </li>';
                $cpt++;
            }
        }


        $htmlhistory.='</ul>';

        $response = new Response(json_encode(array(
            'history' => $htmlhistory,
            'title' => 'Tape #'.$tape->getTapeid()

        )));

        $response->headers->set('Content-Type', 'application/json');

        return $response;
    }


    /**
     * @Route("/tapeMajMulti", name="tape_maj_multi")
     */

    public function tapeMajMulti(Request $request)
    {

        $ids = $request->request->get('listeids');
        $statutId = $request->request->get('statut');
        $serviceId = $request->request->get('service');
        $commentaire = $request->request->get('commentaire');
        $description = $request->request->get('description');

        $em = $this->getDoctrine()->getManager();

        try {
            foreach ($ids as $id) {
                $tape = $this->getDoctrine()
                    ->getRepository(Tape::class)->find($id);

                if ($statutId != 0) {
                    $statut = $this->getDoctrine()
                        ->getRepository(Statut::class)->find($statutId);
                    $tape->setStatutId($statut);
                }
                if ($serviceId != 0 ) {
                    $service = $this->getDoctrine()
                        ->getRepository(Service::class)->find($serviceId);
                    $tape->setServiceId($service);
                }
                if (strlen($commentaire) > 1)
                    $tape->setCommentaire($commentaire);

                if (strlen($description) > 1)
                    $tape->setDescription($description);

                if($statutId) {
                    $arr = json_decode($tape->getHistory(),TRUE);
                    $date = new \DateTime("now");
                    $arr[] = array($statutId => $date);

                    $tape->setHistory(json_encode($arr));
                }

                $em->persist($tape);
            }
            $em->flush();

            $response = new Response(json_encode(array(
                'status' => '200',
                'message' => 'Base de données mise à jour'
            )));
        }

        catch (DBALException $e) {

        $response = new Response(json_encode(array(
        'status' => '500',
        'message' => $e
        )));
        }


        $response->headers->set('Content-Type', 'application/json');
        return $response;


    }

    /**
     * @Route("/removeModel", name="remove_Model")
     */
    public function removeModel(Request $request){
        $id=$request->request->get('id');
        $em = $this->getDoctrine()->getManager();

        /** @var TYPE_NAME $exception */
        try {
            $model=$this->getDoctrine()
                ->getRepository(TapeModele::class)->find($id);

            $em->remove($model);
            $em->flush($model);


            $response = new Response(json_encode(array(
                'status' => '200',
                'message' => 'Base de données mise à jour'
            )));

        }
        catch (DBALException $e) {

            $response = new Response(json_encode(array(
                'status' => '500',
                'message' => $e
            )));
        }


        $response->headers->set('Content-Type', 'application/json');
        return $response;

    }

}