<?php

namespace App\Entity;

use Doctrine\ORM\Mapping as ORM;
use JMS\Serializer\Annotation as Serializer;

/**
 * Tape
 * @ORM\Table(name="tape", indexes={@ORM\Index(name="ref_lot", columns={"LotID"}), @ORM\Index(name="ref_service", columns={"ServiceID"}), @ORM\Index(name="ref_model", columns={"ModeleID"}), @ORM\Index(name="ref_librairie", columns={"LibrairieID"}), @ORM\Index(name="ref_statut", columns={"StatutID"})})
 * @ORM\Entity(repositoryClass=App\Repository\TapeRepository::class)
 *
 * @Serializer\ExclusionPolicy("ALL")
 */
class Tape
{
    /**
     * @var int
     *
     * @ORM\Column(name="TapeID", type="integer", nullable=false)
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     * @Serializer\Expose()
     */
    private $tapeid;

    /**
     * @var string|null
     *
     * @ORM\Column(name="Description", type="string", length=255, nullable=true)
     */
    private $description;

    /**
     * @var string
     *
     * @ORM\Column(name="Label", type="string", length=6, nullable=false)
     * @Serializer\Expose()
     */
    private $label;

    /**
     * @var string|null
     *
     * @ORM\Column(name="NumeroSerie", type="string", length=255, nullable=true)
     */
    private $numeroserie;

    /**
     * @var int|null
     *
     * @ORM\Column(name="Position", type="integer", nullable=true)
     */
    private $position = '0';

    /**
     * @var string|null
     *
     * @ORM\Column(name="Prix", type="decimal", precision=10, scale=0, nullable=true)
     */
    private $prix;

    /**
     * @var string|null
     *
     * @ORM\Column(name="Commentaire", type="string", length=255, nullable=true)
     */
    private $commentaire;

    /**
     * @var \DateTime
     *
     * @ORM\Column(name="DateCreation", type="datetime", nullable=false)
     * @Serializer\Expose()
     */
    private $datecreation;

    /**
     * @var \DateTime|null
     *
     * @ORM\Column(name="DateSuppression", type="datetime", nullable=true)
     */
    private $datesuppression;

    /**
     * @var string
     *
     * @ORM\Column(name="history", type="text", length=0, nullable=false)
     */
    private $history;

    /**
     * @var \Librairie
     *
     * @ORM\ManyToOne(targetEntity="Librairie")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="LibrairieID", referencedColumnName="LibrairieID")
     * })
     * @Serializer\Expose()
     * @Serializer\SerializedName("library")
     */
    private $librairieid;

    /**
     * @var \Lot
     *
     * @ORM\ManyToOne(targetEntity="Lot")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="LotID", referencedColumnName="LotID")
     * })
     */
    private $lotid;

    /**
     * @var \TapeModele
     *
     * @ORM\ManyToOne(targetEntity="TapeModele")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="ModeleID", referencedColumnName="ModelID")
     * })
     * @Serializer\Expose()
     */
    private $modeleid;

    /**
     * @var \Service
     *
     * @ORM\ManyToOne(targetEntity="Service")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="ServiceID", referencedColumnName="ServiceID")
     * })
     * @Serializer\Expose()
     */
    private $serviceid;

    /**
     * @var \Statut
     *
     * @ORM\ManyToOne(targetEntity="Statut")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="StatutID", referencedColumnName="StatutID")
     * })
     * @Serializer\Expose()
     */
    private $statutid;

    public function getId(): ?int
    {
        return $this->getTapeid();
    }

    public function getTapeid(): ?int
    {
        return $this->tapeid;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(?string $description): self
    {
        $this->description = $description;

        return $this;
    }

    public function getLabel(): ?string
    {
        return $this->label;
    }

    public function setLabel(string $label): self
    {
        $this->label = $label;

        return $this;
    }

    public function getNumeroserie(): ?string
    {
        return $this->numeroserie;
    }

    public function setNumeroserie(?string $numeroserie): self
    {
        $this->numeroserie = $numeroserie;

        return $this;
    }

    public function getPosition(): ?int
    {
        return $this->position;
    }

    public function setPosition(?int $position): self
    {
        $this->position = $position;

        return $this;
    }

    public function getPrix(): ?string
    {
        return $this->prix;
    }

    public function setPrix(?string $prix): self
    {
        $this->prix = $prix;

        return $this;
    }

    public function getCommentaire(): ?string
    {
        return $this->commentaire;
    }

    public function setCommentaire(?string $commentaire): self
    {
        $this->commentaire = $commentaire;

        return $this;
    }

    public function getDatecreation(): ?\DateTimeInterface
    {
        return $this->datecreation;
    }

    public function setDatecreation(\DateTimeInterface $datecreation): self
    {
        $this->datecreation = $datecreation;

        return $this;
    }

    public function getDatesuppression(): ?\DateTimeInterface
    {
        return $this->datesuppression;
    }

    public function setDatesuppression(?\DateTimeInterface $datesuppression): self
    {
        $this->datesuppression = $datesuppression;

        return $this;
    }

    public function getHistory(): ?string
    {
        return $this->history;
    }

    public function setHistory(string $history): self
    {
        $this->history = $history;

        return $this;
    }

    public function getLibrairieid(): ?Librairie
    {
        return $this->librairieid;
    }

    public function setLibrairieid(?Librairie $librairieid): self
    {
        $this->librairieid = $librairieid;

        return $this;
    }

    public function getLotid(): ?Lot
    {
        return $this->lotid;
    }

    public function setLotid(?Lot $lotid): self
    {
        $this->lotid = $lotid;

        return $this;
    }

    public function getModeleid(): ?TapeModele
    {
        return $this->modeleid;
    }

    public function setModeleid(?TapeModele $modeleid): self
    {
        $this->modeleid = $modeleid;

        return $this;
    }

    public function getServiceid(): ?Service
    {
        return $this->serviceid;
    }

    public function setServiceid(?Service $serviceid): self
    {
        $this->serviceid = $serviceid;

        return $this;
    }

    public function getStatutid(): ?Statut
    {
        return $this->statutid;
    }

    public function setStatutid(?Statut $statutid): self
    {
        $this->statutid = $statutid;

        return $this;
    }


}
